using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Net;
using System.IO;

namespace EasyImages
{
    public class CameraConnectionFailedException : Exception
    {
        public CameraConnectionFailedException(String error) :base(error)
        {
        }
    }

    public class CameraServerStarted : EventArgs
    {

    }

    public class CameraServerStopped : EventArgs
    {

    }


    public class CameraClient
    {
        Timer t;
        EasyImages.MemoryMappedFile mf;
        EasyImages.MemoryMappedFileView mfv;
        private int updateRate = 100;
        private float fps = 10;
        public delegate void ReceivedFrameEventHandler(object sender, CameraEventArgs e);
        public event ReceivedFrameEventHandler ReceivedFrame;
        public delegate void ReceivedServerStoppedEventHandler(object sender, CameraServerStopped e);
        public event ReceivedServerStoppedEventHandler ServerStopped;
        public delegate void ReceivedServerStartedEventHandler(object sender, CameraServerStarted e);
        public event ReceivedServerStartedEventHandler ServerStarted;

        bool running = false;
        bool connected = false;
        public bool SpawnServer = true;

        private float serverSpeed;
        public float ServerMaxSpeed
        {
            get
            {
                return serverSpeed;
            }
        }

        public float FramesPerSecond
        {
            set
            {
                fps = value;
                updateRate = (int)(1000.0 / fps);
                if (running)
                {
                    t.Interval = updateRate;
                }
            }
            get
            {
                return fps;
            }
        }

        string region;
   
        public CameraClient(String connectionRegion)
        {
            t = new Timer();
            t.Tick += new EventHandler(t_Tick);
            region = connectionRegion;
        }

        void t_Tick(object sender, EventArgs e)
        {
            Image f = GetFrame();
            if (f != null)
            {
                CameraEventArgs ee = new CameraEventArgs(new Bitmap(f));
                this.ReceivedFrame(this, ee);
            }
            else
            {
                CameraEventArgs ee = new CameraEventArgs(GetTestPattern());
                this.ReceivedFrame(this, ee);
            }
        }

        public bool Start()
        {
            if (!running)
            {
                if (Connect())
                {
                    started = false;
                    stopped = false;
                    connected = true;
                    running = true;
                    t.Interval = updateRate;
                    t.Start();
                    return true;
                }
                else return false;
            }
            return false;
        }

        public void Stop()
        {
            if (running)
            {
                connected = true;
                running = false;
                mfv.Dispose();
                mf.Dispose();
                t.Stop();
            }
        }

        bool cameraFailed = false;

        internal void Failed()
        {
            waitForClose = false;
            cameraFailed = true;
            if (cd != null)
            {
                new CameraConnectionFailedException("Client closed the dialoguebox without Sever opening");
//                cd.Dispose(true);
            }
        }

        private void SpawnServerDialogue()
        {
            string dir = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).ToString();
            System.Reflection.Assembly asm = System.Reflection.Assembly.LoadFile( dir.Substring(6) +  @"\EasyImages.dll");
            //System.Resources.ResourceManager rm = new System.Resources.ResourceManager("EasyImages", asm);
            Stream fs = asm.GetManifestResourceStream("EasyImages.CameraServer.exe");
//            if (!System.IO.File.Exists("CameraServer.exe") || System.IO.File."CameraServer.exe"))
//            {
            try
            {
                FileStream filewrite = new FileStream("CameraServer.exe", FileMode.Create); //*/
                ReadWriteStream(fs, filewrite);
            }
            catch (Exception g) { };
//            }
        
        
        }

        public Bitmap GetTestPattern()
        {
            string dir = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).ToString();
            System.Reflection.Assembly asm = System.Reflection.Assembly.LoadFile(dir.Substring(6) + @"\EasyImages.dll");
            Stream fs = asm.GetManifestResourceStream("EasyImages.TestPattern.JPG");
            try
            {

                Font f = new Font(FontFamily.GenericMonospace, 48, FontStyle.Bold);
                string clock = DateTime.Now.Hour.ToString() + ":" + DateTime.Now.Minute.ToString().PadLeft(2, '0') + ":" + DateTime.Now.Second.ToString().PadLeft(2, '0');
                Bitmap b = new Bitmap(fs);
                Graphics g = Graphics.FromImage((Image)b);
                g.DrawString(clock, f, Brushes.DarkGray, 10, 10);//(bmp, 0, 0, newWidth, newHeight);
                return b;
            }
            catch (Exception g) { };
            return null;
        }

        private void ReadWriteStream(Stream readStream, Stream writeStream)
        {
            int Length = 256;
            Byte[] buffer = new Byte[Length];
            
            int bytesRead = readStream.Read(buffer, 0, Length);
           // readStream.
            // write the required bytes
            while (bytesRead > 0)
            {
                writeStream.Write(buffer, 0, bytesRead);
                bytesRead = readStream.Read(buffer, 0, Length);
            }
            readStream.Close();
            writeStream.Close();
        }

        CameraDialogue cd;

        internal void KillDialogueAndRestart()
        {
            if (cd != null)
            {
                cd.Dispose(true);
                this.Start();
            }
            waitForClose = false;
        }

        bool waitForClose;
        bool started = false;
        bool stopped = false;

        private bool Connect()
        {
            mf = MemoryMappedFile.CreateFile(region, MemoryMappedFile.FileAccess.ReadOnly, 131076);
            mfv = mf.CreateView(0, 131076, MemoryMappedFileView.ViewAccess.Read);
            if (GetFrame() == null && SpawnServer)
            {
                mf.Dispose();
                mfv.Dispose();
                mf = null;
                mfv = null;
                SpawnServerDialogue();
                System.Diagnostics.Process p = new System.Diagnostics.Process();
                p.StartInfo.FileName = "CameraServer.exe";
                p.Start();
                if (MessageBox.Show("Waiting for you to start the camera server. \nIn the camera server dialog box start a camera named\"" + region + "\". \nPress OK when the Camera is running, otherwise CANCEL.", "Start Camera", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly).Equals(DialogResult.OK))
                {
                    mf = MemoryMappedFile.CreateFile(region, MemoryMappedFile.FileAccess.ReadOnly, 131076);
                    mfv = mf.CreateView(0, 131076, MemoryMappedFileView.ViewAccess.Read);
                    if (GetFrame() == null)
                    {
                        mf.Dispose();
                        mfv.Dispose();
                        mf = null;
                        mfv = null;
                        return false;
                    }
                    return true;
                }
                return false;
            }

            return true;
        }

        string locker = "";

        public Image GetFrame()
        {
            byte startstop;
            byte[] x = new byte[131071];
            try
            {
                startstop = mfv.ReadByte(0);
                byte[] speed = new byte[4];
                mfv.ReadBytes(speed, 1);
                this.serverSpeed = System.BitConverter.ToSingle(speed, 0);
                mfv.ReadBytes(x, 5);
                if (connected)
                {
                    if (startstop == 1)
                    {
                        stopped = false;
                        if (!started)
                        {
                            started = true;
                            this.ServerStarted(this, new CameraServerStarted());
                        }
                    }
                    else if (startstop == 2)
                    {
                        started = false;
                        if (!stopped)
                        {
                            stopped = true;
                            this.ServerStopped(this, new CameraServerStopped());
                        }
                    }
                    else
                    {
                        stopped = false;
                        started = false;
                    }
                }
                return EasyImages.ImageManipulator.FormattedByteArrayToImage(x);
            }
            catch (Exception SDSS) { return null; }
            
        }
    }
}
