using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Threading;

namespace EasyImages
{

     class API32
    {
        // API32 calls
    //    [DllImport("user32")]
    //    public static extern bool ShowWindow(IntPtr hwnd, int nCmdShow);
        [DllImport("avicap32.dll")]
        public static extern IntPtr capCreateCaptureWindowA(byte[] lpszWindowName, int dwStyle, int x, int y, int nWidth, int nHeight, IntPtr hWndParent, int nID);
        [DllImport("avicap32.dll")]
        public static extern bool capGetDriverDescriptionA(short wDriver, byte[] lpszName, int cbName, byte[] lpszVer, int cbVer);
        [DllImport("User32.dll")]
        public static extern bool SendMessage(IntPtr hWnd, int wMsg, bool wParam, int lParam);
        [DllImport("User32.dll")]
        public static extern bool SendMessage(IntPtr hWnd, int wMsg, short wParam, int lParam);
        [DllImport("User32.dll")]
        public static extern bool SendMessage(IntPtr hWnd, int wMsg, short wParam, FrameEventHandler lParam);
        [DllImport("User32.dll")]
        public static extern bool SendMessage(IntPtr hWnd, int wMsg, int wParam, ref BITMAPINFO lParam);
        [DllImport("User32.dll")]
        public static extern int SetWindowPos(IntPtr hWnd, int hWndInsertAfter, int x, int y, int cx, int cy, int wFlags);
        [DllImport("avicap32.dll")]
        public static extern int capGetVideoFormat(IntPtr hWnd, IntPtr psVideoFormat, int wSize);

        // Constants
        public const int WM_USER = 0x400;
        public const int WS_CHILD = 0x40000000;
        public const int WS_VISIBLE = 0x10000000;
        public const int SWP_NOMOVE = 0x2;
        public const int SWP_NOZORDER = 0x4;
        public const int WM_CAP_DRIVER_CONNECT = WM_USER + 10;
        public const int WM_CAP_DRIVER_DISCONNECT = WM_USER + 11;
        public const int WM_CAP_SET_CALLBACK_FRAME = WM_USER + 5;
        public const int WM_CAP_SET_PREVIEW = WM_USER + 50;
        public const int WM_CAP_SET_PREVIEWRATE = WM_USER + 52;
        public const int WM_CAP_SET_VIDEOFORMAT = WM_USER + 45;
        public const int WM_CAP_CONNECT = 1034;
        public const int WM_CAP_DISCONNECT = 1035;
        public const int WM_CAP_GET_FRAME = 1084;
        public const int WM_CAP_COPY = 1054;

        // Structures
        [StructLayout(LayoutKind.Sequential)]
        public struct VIDEOHDR
        {
            [MarshalAs(UnmanagedType.I4)]
            public int lpData;
            [MarshalAs(UnmanagedType.I4)]
            public int dwBufferLength;
            [MarshalAs(UnmanagedType.I4)]
            public int dwBytesUsed;
            [MarshalAs(UnmanagedType.I4)]
            public int dwTimeCaptured;
            [MarshalAs(UnmanagedType.I4)]
            public int dwUser;
            [MarshalAs(UnmanagedType.I4)]
            public int dwFlags;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
            public int[] dwReserved;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct BITMAPINFOHEADER
        {
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biSize;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biWidth;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biHeight;
            [MarshalAs(UnmanagedType.I2)]
            public short biPlanes;
            [MarshalAs(UnmanagedType.I2)]
            public short biBitCount;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biCompression;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biSizeImage;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biXPelsPerMeter;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biYPelsPerMeter;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biClrUsed;
            [MarshalAs(UnmanagedType.I4)]
            public Int32 biClrImportant;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct BITMAPINFO
        {
            [MarshalAs(UnmanagedType.Struct, SizeConst = 40)]
            public BITMAPINFOHEADER bmiHeader;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 1024)]
            public Int32[] bmiColors;
        }

        public delegate void FrameEventHandler(IntPtr lwnd, IntPtr lpVHdr);

        // Public methods
        public static object GetStructure(IntPtr ptr, ValueType structure)
        {
            return Marshal.PtrToStructure(ptr, structure.GetType());
        }

        public static object GetStructure(int ptr, ValueType structure)
        {
            return GetStructure(new IntPtr(ptr), structure);
        }

        public static void Copy(IntPtr ptr, byte[] data)
        {
            Marshal.Copy(ptr, data, 0, data.Length);
        }

        public static void Copy(int ptr, byte[] data)
        {
            Copy(new IntPtr(ptr), data);
        }

        public static int SizeOf(object structure)
        {
            return Marshal.SizeOf(structure);
        }
    }

    public class Camera
    {
 /*       /// <summary>
        ///  The Default Constructor.  If issues arise, use the other constructor
        /// </summary>
        public Camera()
        {
            parentControl = Application.OpenForms[0];
            mControlPtr = Application.OpenForms[0].Handle;
            FramesPerSecond = 24;
            //lastImage2 = new Bitmap(10,10);
        } */

        /// <summary>
        /// Standard Constructor
        /// </summary>
        /// <param name="ip">The handle to the main window</param>
        public Camera(Form f)
        {
            timerDelegate = new TimerCallback(this.getFrame);
            parentControl = f;
            mControlPtr = f.Handle;
            FramesPerSecond = 10;
            //lastImage2 = new Bitmap(10,10);
        }

        private void workRoutine()
        {
        }

        String locker = "lock!";
        Form parentControl;
        bool memorymapped = false;
        bool running = false;
        bool noCamera = false;
        TimerCallback timerDelegate;
        public delegate void ReceivedFrameEventHandler(object sender, CameraEventArgs e);
        public event ReceivedFrameEventHandler ReceivedFrame;
        private API32.FrameEventHandler mFrameEventHandler;
        IntPtr lwndC; // Holds the unmanaged handle of the control
        private IntPtr mControlPtr; // Holds the managed pointer of the control
        private Bitmap lastImage;
//        private Bitmap lastImage2;
        private bool initialized = false;
        public Bitmap CurrentFrame
        {
            get
            {
                if (memorymapped) return null;
                if (running)
                {
                    //getFrame();
                    lock (locker)
                    {
                        //getFrame();
                        return lastImage.Clone() as Bitmap;
                    }
                }
                else if(initialized){
                    getFrame();
                    lock (locker)
                    {
                        return lastImage.Clone() as Bitmap;
                    }
                }

                else
                {
                    Initialize();
                    getFrame();
                    lock (locker)
                    {
                        //getFrame();
                        return lastImage.Clone() as Bitmap;
                    }
                }
            }
        }
        public int Width = 640;
        public int Height = 480;
        public int CameraID = 0;
        private int updateRate = 100;
        private float fps;
        public float FramesPerSecond 
        {
            set
            {
                fps = value;
                updateRate = (int) (1000.0 / fps);
                if (running)
                {
                    grabFrameTimer.Interval = updateRate;
                    returnFrame.Change(0, updateRate);
//                    this.capPreviewRate(this.lwndC, (short)updateRate);
                }
            }
            get
            {
                return fps;
            }
        }

        private void Initialize()
        {
            initialized = true;
            byte[] lpszName = new byte[100];
            byte[] lpszVer = new byte[100];
            API32.capGetDriverDescriptionA((short)CameraID, lpszName, 100, lpszVer, 100);
            this.lwndC = API32.capCreateCaptureWindowA(lpszName, API32.WS_CHILD, 0, 0, 1, 1, mControlPtr, 0);
            if (this.capDriverConnect(this.lwndC, (short)CameraID))
            {
                API32.BITMAPINFO bitmapinfo = new API32.BITMAPINFO();
                lastImage = new Bitmap(Width, Height);
                bitmapinfo.bmiHeader.biSize = API32.SizeOf(bitmapinfo.bmiHeader);
                bitmapinfo.bmiHeader.biWidth = Width;
                bitmapinfo.bmiHeader.biHeight = Height;
                bitmapinfo.bmiHeader.biPlanes = 1;
                bitmapinfo.bmiHeader.biBitCount = 24;
                this.capSetVideoFormat(this.lwndC, ref bitmapinfo, API32.SizeOf(bitmapinfo));
                this.mFrameEventHandler = new API32.FrameEventHandler(FrameCallBack);
                this.capSetCallbackOnFrame(this.lwndC, this.mFrameEventHandler);
            }
            else
            {
                Stop();
                throw new Exception("Camera unavailable!");
            }
        }

        /// <summary>
        ///  Closes the web camera.
        /// </summary>
        public void Stop()
        {
            if (memorymapped)
            {
                this.testPattern = false;
                this.grabFrameTimer.Stop();
                stopped = true;

                if (!noCamera) getFrame();
                else
                {
                    ProcessTestFrame();
                }
                mf.Dispose();
                mfv.Dispose();
                memorymapped = false;
                mf = null;
                mfv = null;
                this.capDriverDisconnect(this.lwndC);
            }
            else if (running)
            {
                running = false;
                this.returnFrame.Change(-1, 0);
                this.grabFrameTimer.Stop();
                this.capDriverDisconnect(this.lwndC);
            }
            else if (initialized)
            {
                this.capDriverDisconnect(this.lwndC);
                initialized = false;
            }
        }

        bool testPattern = false;
        bool stopped = true;
        int starting;
        MemoryMappedFile mf;
        MemoryMappedFileView mfv;
        public void StartFileMapping(string name)
        {
            this.noCamera = false;
            if (initialized || running || memorymapped)
            {
                Stop();
            }
            stopped = false;
            starting = ((int) FramesPerSecond) + 10;
            memorymapped = true;
            mf = MemoryMappedFile.CreateFile(name, MemoryMappedFile.FileAccess.ReadWrite, 131076);
            mfv = mf.CreateView(0, 131076, MemoryMappedFileView.ViewAccess.ReadWrite);
            grabFrameTimer = new System.Windows.Forms.Timer();
            grabFrameTimer.Tick += new EventHandler(grabFrameTimerThing_Tick);
            grabFrameTimer.Interval = updateRate;
            try
            {
                DisplayWorkMapped();
            }
            catch (Exception e) {this.noCamera = true; grabFrameTimer.Start(); }
        }

        public void StartMappedTestPattern(string name){
            testPattern = true;
            StartFileMapping(name);
        }

        private Bitmap GetTestPattern()
        {
            string dir = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).ToString();
            System.Reflection.Assembly asm = System.Reflection.Assembly.LoadFile(dir.Substring(6) + @"\EasyImages.dll");
            //System.Resources.ResourceManager rm = new System.Resources.ResourceManager("EasyImages", asm);
            Stream fs = asm.GetManifestResourceStream("EasyImages.TestPattern.JPG");
            try
            {
                Font f = new Font(FontFamily.GenericMonospace, 48, FontStyle.Bold);
                string clock = DateTime.Now.Hour.ToString() + ":" + DateTime.Now.Minute.ToString().PadLeft(2, '0') + ":" + DateTime.Now.Second.ToString().PadLeft(2, '0');
                Bitmap b = new Bitmap(fs);
                Graphics g = Graphics.FromImage((Image)b);
                g.DrawString(clock, f, Brushes.DarkGray, 10, 10);//(bmp, 0, 0, newWidth, newHeight);
                return (Bitmap) EasyImages.ImageGenerator.Resize(this.Width, this.Height, b);
                //return b;
            }
            catch (Exception g) { };
            return null;
        }

        System.Windows.Forms.Timer grabFrameTimer;
        System.Threading.Timer returnFrame;
        /// <summary>
        ///  start the web camera
        /// </summary>
        public void Start()
        {
            if (initialized || running || memorymapped)
            {
                Stop();
            }
            grabFrameTimer = new System.Windows.Forms.Timer();
            grabFrameTimer.Tick += new EventHandler(grabFrameTimerThing_Tick);
            grabFrameTimer.Interval = updateRate;
            returnFrame = new System.Threading.Timer(new TimerCallback(returnFrame_Tick));
            running = true;
            DisplayWork();
        }

        void returnFrame_Tick(object sender)
        {
            lock (locker)
            {
                Bitmap bmp = lastImage.Clone() as Bitmap;
                CameraEventArgs ev = new CameraEventArgs(bmp);
                this.ReceivedFrame(this, ev);
            }
          }

        void grabFrameTimerThing_Tick(object sender, EventArgs e)
        {
            if (!noCamera) getFrame();
            else
            {
                ProcessTestFrame();       
            }
        }

        private void ProcessTestFrame()
        {
            byte[] speed = System.BitConverter.GetBytes(this.fps);
            byte[] imageData = (new JPEG(this.GetTestPattern(), 90)).GetByteArray();
            if (starting > 0)
            {
                mfv.WriteByte((byte)1, 0);
                starting--;
                mfv.WriteBytes(speed, 1);
            }
            else if (stopped)
            {
                mfv.WriteByte((byte)2, 0);
                mfv.WriteByte(0, 1);
                mfv.WriteByte(0, 2);
                mfv.WriteByte(0, 3);
                mfv.WriteByte(0, 4);
            }
            else
            {
                mfv.WriteByte((byte)0, 0);
                mfv.WriteBytes(speed, 1);
            }
            mfv.WriteBytes(imageData, 5);
        }

        public string GetCameraName(short id)
        {
            byte[] lpszName = new byte[100];
            byte[] lpszVer = new byte[100];
            API32.capGetDriverDescriptionA(id, lpszName, 100, lpszVer, 100);

            return System.Text.Encoding.ASCII.GetString(lpszName).Replace("\0", "");
        }

        private void DisplayWorkMapped()
        {
            if (testPattern)
            {
                throw new Exception("Camera unavailable!");
            }

            byte[] lpszName = new byte[100];
            byte[] lpszVer = new byte[100];
            API32.capGetDriverDescriptionA((short)CameraID, lpszName, 100, lpszVer, 100);
            this.lwndC = API32.capCreateCaptureWindowA(lpszName, API32.WS_CHILD, 0, 0, 1, 1, mControlPtr, 0);
            if (this.capDriverConnect(this.lwndC, (short)CameraID))
            {
                API32.BITMAPINFO bitmapinfo = new API32.BITMAPINFO();
                lastImage = new Bitmap(Width, Height);
                bitmapinfo.bmiHeader.biSize = API32.SizeOf(bitmapinfo.bmiHeader);
                bitmapinfo.bmiHeader.biWidth = Width;
                bitmapinfo.bmiHeader.biHeight = Height;
                bitmapinfo.bmiHeader.biPlanes = 1;
                bitmapinfo.bmiHeader.biBitCount = 24;
                this.capSetVideoFormat(this.lwndC, ref bitmapinfo, API32.SizeOf(bitmapinfo));
                this.mFrameEventHandler = new API32.FrameEventHandler(FrameCallBack);
                this.capSetCallbackOnFrame(this.lwndC, this.mFrameEventHandler);
                grabFrameTimer.Start();
            }
            else
            {
                //Stop();
                throw new Exception("Camera unavailable!");
            }
        }

        private void DisplayWork()
        {
            
           // parentControl.BringToFront();
            byte[] lpszName = new byte[100];
            byte[] lpszVer = new byte[100];
            API32.capGetDriverDescriptionA((short) CameraID, lpszName, 100, lpszVer, 100);
            this.lwndC = API32.capCreateCaptureWindowA(lpszName, API32.WS_CHILD, 0, 0, 1, 1, mControlPtr, 0);
            if (this.capDriverConnect(this.lwndC, (short)CameraID))
            {
                //this.capPreviewRate(this.lwndC, (short)updateRate);
                //this.capPreview(this.lwndC, true);
                API32.BITMAPINFO bitmapinfo = new API32.BITMAPINFO();
                lastImage = new Bitmap(Width, Height);
                bitmapinfo.bmiHeader.biSize = API32.SizeOf(bitmapinfo.bmiHeader);
                bitmapinfo.bmiHeader.biWidth = Width;
                bitmapinfo.bmiHeader.biHeight = Height;
                bitmapinfo.bmiHeader.biPlanes = 1;
                bitmapinfo.bmiHeader.biBitCount = 24;
                this.capSetVideoFormat(this.lwndC, ref bitmapinfo, API32.SizeOf(bitmapinfo));
                this.mFrameEventHandler = new API32.FrameEventHandler(FrameCallBack);
                this.capSetCallbackOnFrame(this.lwndC, this.mFrameEventHandler);
                grabFrameTimer.Start();
                returnFrame.Change(0, updateRate);
                //grabFrameTimer.Change(0, updateRate);
            }
            else
            {
                Stop();
                throw new Exception("Camera unavailable!");
            }
        }

        private bool capDriverConnect(IntPtr lwnd, short i)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_DRIVER_CONNECT, i, 0);
        }

        public delegate void SetControlImage(PictureBox icontrol, Bitmap image);

        public void RecievedImageToControl(PictureBox icontrol, Image image)
        {

            if (icontrol.InvokeRequired)
            {
                SetControlImage theDelegate = new SetControlImage(RecievedImageToControl);
                this.parentControl.BeginInvoke(theDelegate, new object[] { icontrol, image });
            }
            else
            {
                icontrol.Image = image;
            }
        } 


        private void getFrame()
        {
            API32.SendMessage(this.lwndC, API32.WM_CAP_GET_FRAME, 0, 0);
        }

        private void getFrame(Object c)
        {
            API32.SendMessage(this.lwndC, API32.WM_CAP_GET_FRAME, 0, 0);
        }
        private bool capDriverDisconnect(IntPtr lwnd)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_DRIVER_DISCONNECT, 0, 0);
        }

        private bool capPreview(IntPtr lwnd, bool f)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_PREVIEW, f, 0);
        }

        private bool capPreviewRate(IntPtr lwnd, short wMS)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_PREVIEWRATE, wMS, 0);
        }

        private bool capSetCallbackOnFrame(IntPtr lwnd, API32.FrameEventHandler lpProc)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_CALLBACK_FRAME, 0, lpProc);
        }

        private bool capSetVideoFormat(IntPtr hCapWnd, ref API32.BITMAPINFO BmpFormat, int CapFormatSize)
        {
            return API32.SendMessage(hCapWnd, API32.WM_CAP_SET_VIDEOFORMAT, CapFormatSize, ref BmpFormat);
        }


        private void FrameCallBack(IntPtr lwnd, IntPtr lpVHdr)
        {
            lock (locker)
            {
                unsafe
                {
                    API32.VIDEOHDR videoHeader = new API32.VIDEOHDR();
                    videoHeader = (API32.VIDEOHDR)API32.GetStructure(lpVHdr, videoHeader);
                    System.Drawing.Imaging.BitmapData bmpData = lastImage.LockBits(new Rectangle(0, 0, Width, Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
                    int length = videoHeader.dwBytesUsed;

                    byte* imgPtr = (byte*)(bmpData.Scan0);
                    byte* imgPtr2 = (byte*)(videoHeader.lpData);
                    for (int i = 0; i < length; i++)
                    {
                        *imgPtr = *imgPtr2;
                        imgPtr++;
                        imgPtr2++;
                    }

                    lastImage.UnlockBits(bmpData);
                    lastImage.RotateFlip(RotateFlipType.RotateNoneFlipY);
                    if (memorymapped)
                    {   
                        
                        byte[] speed = System.BitConverter.GetBytes(this.fps);
                        byte[] imageData = (new JPEG(lastImage, 90)).GetByteArray();
          //              Console.WriteLine((new JPEG(lastImage, 96)).GetByteArray().Length);
         //               byte[] data = new byte[speed.Length + imageData.Length];
         //               System.Array.Copy(speed,0, data, 0, speed.Length);
         //               System.Array.Copy(imageData, 0, data, 4, imageData.Length);
                        if (starting > 0)
                        {
                            mfv.WriteByte((byte)1, 0);
                            starting--;
                            mfv.WriteBytes(speed, 1);
                        }
                        else if (stopped)
                        {
                            mfv.WriteByte((byte)2, 0);
                            mfv.WriteByte(0, 1);
                            mfv.WriteByte(0, 2);
                            mfv.WriteByte(0, 3);
                            mfv.WriteByte(0, 4);
                        }
                        else
                        {
                            mfv.WriteByte((byte)0, 0);
                            mfv.WriteBytes(speed, 1);
                        }
//                       mfv.WriteBytes(speed, 1);
                       mfv.WriteBytes(imageData, 5);
//                        Console.WriteLine(mfv.ReadByte(0));
                    }
                }
            }
        }
    }

    public class CameraEventArgs : EventArgs
    {

        private System.Drawing.Bitmap bitMap;

        public CameraEventArgs(System.Drawing.Bitmap bmp)
        {
            bitMap = bmp;

        }

        public System.Drawing.Bitmap Bitmap
        {

            get
            {

                return bitMap;

            }

        }

    }

}