/// OLD CAMERA
/// 
/*
public class CameraEventArgs : System.EventArgs
{
    private System.Drawing.Bitmap m_Image;
    private ulong m_FrameNumber = 0;

    public CameraEventArgs()
    {
    }

    /// <summary>
    ///  WebCamImage
    ///  This is the image returned by the web camera capture
    /// </summary>
    public System.Drawing.Bitmap CameraImage
    {
        get
        { return m_Image; }

        set
        { m_Image = value; }
    }

    /// <summary>
    /// FrameNumber
    /// Holds the sequence number of the frame capture
    /// </summary>
    public ulong FrameNumber
    {
        get
        { return m_FrameNumber; }

        set
        { m_FrameNumber = value; }
    }
}

public class Camera : System.Windows.Forms.UserControl
{

    private System.ComponentModel.IContainer components;
    private System.Windows.Forms.Timer timer1;
    public int TimeToCaptureInMilliseconds = 100;
    public int imageWidth = 320;
    public int imageHeight = 240;
    private int mCapHwnd;
    private ulong m_FrameNumber = 0;

    // global variables to make the video capture go faster
    private CameraEventArgs x = new CameraEventArgs();
    private IDataObject tempObj;
    private System.Drawing.Image tempImg;
    private bool bStopped = true;

    // event delegate
    public delegate void CameraEventHandler(object source, CameraEventArgs e);
    // fired when a new image is captured
    public event CameraEventHandler ImageCaptured;

    //    public delegate void CameraVideoHandler(long lwnd, long lpVHdr);
    //    public event CameraVideoHandler videoCaptured;

    #region API Declarations

    [DllImport("user32", EntryPoint = "SendMessage")]
    public static extern int SendMessage(int hWnd, uint Msg, int wParam, int lParam);

    [DllImport("avicap32.dll", EntryPoint = "capCreateCaptureWindowA")]
    public static extern int capCreateCaptureWindowA(string lpszWindowName, int dwStyle, int X, int Y, int nWidth, int nHeight, int hwndParent, int nID);

    //      [DllImport("avicap32.dll", EntryPoint = "capSetCallbackOnVideoStream")]
    //      public static extern int capSetCallbackOnVideoStream(int hWndCap, CameraVideoHandler functionPointer);

    [DllImport("user32", EntryPoint = "OpenClipboard")]
    public static extern int OpenClipboard(int hWnd);

    [DllImport("user32", EntryPoint = "EmptyClipboard")]
    public static extern int EmptyClipboard();

    [DllImport("user32", EntryPoint = "CloseClipboard")]
    public static extern int CloseClipboard();

    #endregion

    #region API Constants

    public const int WM_USER = 1024;

    public const int WM_CAP_CONNECT = 1034;
    public const int WM_CAP_DISCONNECT = 1035;
    public const int WM_CAP_GET_FRAME = 1084;
    public const int WM_CAP_COPY = 1054;

    public const int WM_CAP_START = WM_USER;

    public const int WM_CAP_DLG_VIDEOFORMAT = WM_CAP_START + 41;
    public const int WM_CAP_DLG_VIDEOSOURCE = WM_CAP_START + 42;
    public const int WM_CAP_DLG_VIDEODISPLAY = WM_CAP_START + 43;
    public const int WM_CAP_GET_VIDEOFORMAT = WM_CAP_START + 44;
    public const int WM_CAP_SET_VIDEOFORMAT = WM_CAP_START + 45;
    public const int WM_CAP_DLG_VIDEOCOMPRESSION = WM_CAP_START + 46;
    public const int WM_CAP_SET_PREVIEW = WM_CAP_START + 50;

    #endregion

    public Camera()
    {
        this.components = new System.ComponentModel.Container();
        // InitializeComponent();
        this.timer1 = new System.Windows.Forms.Timer(this.components);

        this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
        //   InitializeComponent();
    }

    /// <summary>
    /// Override the class's finalize method, so we can stop
    /// the video capture on exit
    /// </summary>
    ~Camera()
    {
        this.Stop();
    }

   #region Component Designer generated code
    #endregion

    #region Start and Stop Capture Functions
    public delegate void DelegateABC();
    /// <summary>
    /// Starts the video capture
    /// </summary>
    /// <param name="FrameNumber">the frame number to start at. 
    /// Set to 0 to let the control allocate the frame number</param>
    public void Start(ulong FrameNum)
    {
        try
        {
            // for safety, call stop, just in case we are already running
            this.Stop();

            if (!initialized) Initialize();
            m_FrameNumber = FrameNum;
            this.timer1.Interval = TimeToCaptureInMilliseconds;
            bStopped = false;
            this.timer1.Start();
        }

        catch (Exception excep)
        {
            MessageBox.Show("An error ocurred while starting the video capture. Check that your webcamera is connected properly and turned on.\r\n\n" + excep.Message);
            this.Stop();
        }
    }

    bool initialized = false;
    private void Initialize()
    {
        mCapHwnd = capCreateCaptureWindowA("WebCap", 0, 0, 0, imageWidth, imageHeight, this.Handle.ToInt32(), 0);
        Application.DoEvents();
        SendMessage(mCapHwnd, WM_CAP_CONNECT, 0, 0);
        SendMessage(mCapHwnd, WM_CAP_SET_PREVIEW, 0, 0);
        initialized = true;
    }


    public void CallBackFunction(long lwnd, long lpVHdr)
    {
        Console.WriteLine("here");
    }

    /// <summary>
    /// Grabs a frame from the camera and returns it as an image.
    /// </summary>
    /// <returns></returns>
    public Image CaptureFrame()
    {
        Image i = null;
        if (!initialized) Initialize();
        SendMessage(mCapHwnd, WM_CAP_GET_FRAME, 0, 0);
        SendMessage(mCapHwnd, WM_CAP_COPY, 0, 0);
        if (ImageCaptured != null)
        {
            tempObj = Clipboard.GetDataObject();
            tempImg = (System.Drawing.Bitmap)tempObj.GetData(System.Windows.Forms.DataFormats.Bitmap);
            i = tempImg.GetThumbnailImage(imageWidth, imageHeight, null, System.IntPtr.Zero);
        }
        return i;
    }

    /// <summary>
    /// Stops the video capture
    /// </summary>
    public void Stop()
    {
        try
        {
            // stop the timer
            bStopped = true;
            this.timer1.Stop();

            // disconnect from the video source
            Application.DoEvents();
            SendMessage(mCapHwnd, WM_CAP_DISCONNECT, 0, 0);
            initialized = false;
        }

        catch (Exception excep)
        { // don't raise an error here.
        }

    }

    #endregion

    #region Video Capture Code

    /// <summary>
    /// Capture the next frame from the video feed
    /// </summary>
    private void timer1_Tick(object sender, System.EventArgs e)
    {
        try
        {
            // pause the timer
            this.timer1.Stop();

            // get the next frame;
            SendMessage(mCapHwnd, WM_CAP_GET_FRAME, 0, 0);

            // copy the frame to the clipboard
            SendMessage(mCapHwnd, WM_CAP_COPY, 0, 0);

            // paste the frame into the event args image
            if (ImageCaptured != null)
            {
                // get from the clipboard
                tempObj = Clipboard.GetDataObject();
                tempImg = (System.Drawing.Bitmap)tempObj.GetData(System.Windows.Forms.DataFormats.Bitmap);
                x.CameraImage = new Bitmap(tempImg.GetThumbnailImage(imageWidth, imageHeight, null, System.IntPtr.Zero));


                // raise the event
                this.ImageCaptured(this, x);
            }

            timer1.Interval = this.TimeToCaptureInMilliseconds;
            // restart the timer
            Application.DoEvents();
            if (!bStopped)
                this.timer1.Start();
        }

        catch (Exception excep)
        {
            MessageBox.Show("An error ocurred while capturing the video image. The video capture will now be terminated.\r\n\n" + excep.Message);
            this.Stop(); // stop the process
        }
    }

    #endregion

} */