using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using SKYPE4COMLib;

namespace MEdiaSpaceCore
{
    public class AudioSource
    {
        // Callback bits so the hosting UI can get notification about call status changes
        public delegate void AudioSourceStatusChangedHandler(bool talking);
        public event AudioSourceStatusChangedHandler AudioSourceStatusChanged;

        // Internal variables
        private String _sinkUsername;
        private Call _skypeCall;

        private static Skype _skypeObject = new SkypeClass();

        public AudioSource(String sinkUsername)
        {
            // Set up the locals
            _sinkUsername = sinkUsername;
            _skypeCall = null;

            // Start the client, if necessary
            // (better to do this here than waiting until we're trying to call it...)
            this.Reconnect();

            // Set up a callback to pick up call status changes
            _skypeObject.CallStatus += new _ISkypeEvents_CallStatusEventHandler(_skypeObject_CallStatus);

            // Set up a callback to monitor for user status changes, too
            _skypeObject.UserStatus += new _ISkypeEvents_UserStatusEventHandler(_skypeObject_UserStatus);
        }

        public void Start()
        {
            // Double-check to make sure the client's still up and running
            this.Reconnect();

            // PlaceCall fails if we're not shown to be online
            if (_skypeObject.CurrentUserStatus == TUserStatus.cusOffline)
                _skypeObject.ChangeUserStatus(TUserStatus.cusOnline);

            // Create a user object to represent the caller destination)
            User sinkUser = _skypeObject.get_User(_sinkUsername);
            if (sinkUser == null)
            {
                Console.WriteLine("Couldn't find the Skype user " + _sinkUsername);
                return;
            }

            // Place the call!
            _skypeCall = _skypeObject.PlaceCall(sinkUser.Handle, null, null, null);
        }

        public void Stop()
        {
            if (_skypeCall != null)
                _skypeCall.Finish();
        }

        void _skypeObject_CallStatus(Call pCall, TCallStatus status)
        {
            if (pCall.Id == _skypeCall.Id && AudioSourceStatusChanged != null)
            {
                if (status == TCallStatus.clsInProgress)
                    AudioSourceStatusChanged(true);
                else if (status == TCallStatus.clsBusy ||
                         status == TCallStatus.clsCancelled ||
                         status == TCallStatus.clsFailed ||
                         status == TCallStatus.clsFinished ||
                         status == TCallStatus.clsMissed ||
                         status == TCallStatus.clsRefused)
                {
                    AudioSourceStatusChanged(false);
                    _skypeCall.Finish();
                    _skypeCall = null;
                }
            }
        }

        void _skypeObject_UserStatus(TUserStatus status)
        {
            // If we fell off, then reconnect!
            if (status == TUserStatus.cusOffline)
            {
                Reconnect();
            }
        }

        public bool IsTalking()
        {
            if (_skypeObject == null || _skypeCall == null)
                return false;
            else
                return (_skypeObject.CurrentUserStatus == TUserStatus.cusOnline &&
                        _skypeCall.Status == TCallStatus.clsInProgress);
        }

        // Helper function to make sure the client is up and running
        private void Reconnect()
        {
            while (!_skypeObject.Client.IsRunning)
            {
                try
                {
                    _skypeObject.Client.Start(true, true);
                }
                catch (COMException) { }
            }
        }
    }
}
