using GT.Net;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;

namespace MEdiaSpaceCore
{
    public class Variables
    {
        // Callback bits so the hosting UI can be notified if the variables change
        public delegate void VideoConfigurationChangeHandler(Constants.VIDEO_PROTOCOLS protocol,
                                                             int compression,
                                                             double frameRateSlow,
                                                             double frameRateFast);
        public event VideoConfigurationChangeHandler HomeVideoConfigurationChanged = null;
        public event VideoConfigurationChangeHandler OfficeVideoConfigurationChanged = null;

        public delegate void MessageChangeHandler(String message);
        public event MessageChangeHandler HomeMessageChanged = null;
        public event MessageChangeHandler OfficeMessageChanged = null;

        public delegate void NoteTextChangeHandler(String noteText);
        public event NoteTextChangeHandler HomeNoteTextChanged = null;

        public delegate void SkypeHandleChangeHandler(String skypeHandle);
        public event SkypeHandleChangeHandler HomeSkypeHandleChanged = null;
        public event SkypeHandleChangeHandler OfficeSkypeHandleChanged = null;

        public delegate void StatusChangeHandler(String status);
        public event StatusChangeHandler HomeStatusChanged = null;

        public delegate void DoorActuationHandler();
        public event DoorActuationHandler DoorActuated = null;

        public delegate void DoorActuationAvailableHandler(bool available);
        public event DoorActuationAvailableHandler DoorActuationAvailable = null;

        public delegate void DoorbellStatusChangeHandler(bool status);
        public event DoorbellStatusChangeHandler DoorbellStatusChanged = null;

        // Shared Dictionary naming keys -- for keys associated with notifications
        private const String DOOR_ACTUATION_AVAILABLE_NAME = "dooractutationavailable";
        private const String DOORBELL_STATUS = "doorbellstatus";
        private const String HOME_COMPRESSION_NAME = "homecompression";
        private const String HOME_DOOR_ACTUATE_NAME = "homedooractuate";
        private const String HOME_FRAME_RATE_FAST_NAME = "homeframeratefast";
        private const String HOME_FRAME_RATE_SLOW_NAME = "homeframerateslow";
        private const String HOME_MESSAGE_NAME = "homemessage";
        private const String HOME_NOTE_TEXT_NAME = "homenotetext";
        private const String HOME_PROTOCOL_NAME = "homeprotocol";
        private const String HOME_SKYPE_HANDLE_NAME = "homeskypehandle";
        private const String HOME_STATUS_NAME = "homestatus.transient";
        private const String OFFICE_COMPRESSION_NAME = "officecompression";
        private const String OFFICE_FRAME_RATE_FAST_NAME = "officeframeratefast";
        private const String OFFICE_FRAME_RATE_SLOW_NAME = "officeframerateslow";
        private const String OFFICE_MESSAGE_NAME = "officemessage";
        private const String OFFICE_NETWORK_PORT_NAME = "officenetworkport";
        private const String OFFICE_PROTOCOL_NAME = "officeprotocol";
        private const String OFFICE_SKYPE_HANDLE_NAME = "officeskypehandles";

        // GT# Locals
        IObjectStream _objStream;

        // Transient shared variable container
        private Dictionary<String, String> _transients;
        
        // Echo prevention
        private List<String> _localsToIgnore;
        private List<String> _remotesToIgnore;


        //
        // Lifecycle
        //
        public Variables()
        {
            // Initialize our transient variable container
            _transients = new Dictionary<String, String>();
            _transients[DOOR_ACTUATION_AVAILABLE_NAME] = false.ToString();
            _transients[DOORBELL_STATUS] = false.ToString();
            _transients[HOME_MESSAGE_NAME] = null;
            _transients[HOME_NOTE_TEXT_NAME] = null;
            _transients[HOME_SKYPE_HANDLE_NAME] = null;
            _transients[HOME_STATUS_NAME] = null;
            _transients[OFFICE_MESSAGE_NAME] = null;
            _transients[OFFICE_SKYPE_HANDLE_NAME] = null;

            // Initialize our echo-prevention lists
            _localsToIgnore = new List<String>();
            _remotesToIgnore = new List<String>();

            // Subscribe to the settings for local changes
            Properties.Settings.Default.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Default_PropertyChanged);
        }

        //
        // Variable accessors
        //
        public bool doorActuationAvailable
        {
            get { return Convert.ToBoolean(_transients[DOOR_ACTUATION_AVAILABLE_NAME]); }
            set
            {
                _transients[DOOR_ACTUATION_AVAILABLE_NAME] = value.ToString();

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(DOOR_ACTUATION_AVAILABLE_NAME, value.ToString()));
            }
        }

        public int doorbellPort
        {
            get { return Properties.Settings.Default.doorbellport; }
        }

        public bool doorbellStatus
        {
            get { return Convert.ToBoolean(_transients[DOORBELL_STATUS]); }
            set
            {
                _transients[DOORBELL_STATUS] = value.ToString();

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(DOORBELL_STATUS, value.ToString()));
            }
        }

        public double doorMotorDelay
        {
            get { return Properties.Settings.Default.doormotordelay; }
        }

        public int doorMotorPort
        {
            get { return Properties.Settings.Default.doormotorport; }
        }

        public int homeCompression
        {
            get { return Properties.Settings.Default.homecompression; }
        }

        public bool homeConnectOnStartup
        {
            get { return Properties.Settings.Default.homeconnectonstartup; }
        }

        public bool homeDoorbellFlash
        {
            get { return Properties.Settings.Default.homedoorbellflash; }
        }

        public double homeDoorbellFlashDelay
        {
            get { return Properties.Settings.Default.homedoorbellflashdelay; }
        }

        public bool homeDoorbellFocus
        {
            get { return Properties.Settings.Default.homedoorbellfocus; }
        }

        public bool homeDoorbellSound
        {
            get { return Properties.Settings.Default.homedoorbellsound; }
        }

        public String homeDoorbellSoundFile
        {
            get { return Properties.Settings.Default.homedoorbellsoundfile; }
        }

        public double homeFrameRateFast
        {
            get { return Properties.Settings.Default.homeframeratefast; }
        }

        public double homeFrameRateSlow
        {
            get { return Properties.Settings.Default.homeframerateslow; }
        }

        public String homeMessage
        {
            get { return _transients[HOME_MESSAGE_NAME]; }
            set
            {
                _transients[HOME_MESSAGE_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(HOME_MESSAGE_NAME, value));
            }
        }

        public bool homeMotionSensorFlash
        {
            get { return Properties.Settings.Default.homemotionsensorflash; }
        }

        public double homeMotionSensorFlashDelay
        {
            get { return Properties.Settings.Default.homemotionsensorflashdelay; }
        }

        public bool homeMotionSensorFocus
        {
            get { return Properties.Settings.Default.homemotionsensorfocus; }
        }

        public bool homeMotionSensorSound
        {
            get { return Properties.Settings.Default.homemotionsensorsound; }
        }

        public String homeMotionSensorFile
        {
            get { return Properties.Settings.Default.homemotionsensorsoundfile; }
        }

        public String homeNoteText
        {
            get { return _transients[HOME_NOTE_TEXT_NAME]; }
            set
            {
                _transients[HOME_NOTE_TEXT_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(HOME_NOTE_TEXT_NAME, value));
            }
        }

        public Constants.VIDEO_PROTOCOLS homeProtocol
        {
            get { return (Constants.VIDEO_PROTOCOLS)Properties.Settings.Default.homeprotocol; }
        }

        public String homeSkypeHandle
        {
            get { return _transients[HOME_SKYPE_HANDLE_NAME]; }
            set
            {
                _transients[HOME_SKYPE_HANDLE_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(HOME_SKYPE_HANDLE_NAME, value));
            }
        }

        public String homeStatus
        {
            get { return _transients[HOME_STATUS_NAME]; }
            set
            {
                _transients[HOME_STATUS_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(HOME_STATUS_NAME, value));
            }
        }

        public bool interfaceKitFirst
        {
            get { return Properties.Settings.Default.interfacekitfirst; }
        }

        public int interfaceKitSerial
        {
            get { return Properties.Settings.Default.interfacekitserial; }
        }

        public int motionSensorPort
        {
            get { return Properties.Settings.Default.motionsensorport; }
        }

        public int motionSensorSensitivity
        {
            get { return Properties.Settings.Default.motionsensorsensitivity; }
        }

        public int officeCompression
        {
            get { return Properties.Settings.Default.officecompression; }
        }

        public double officeFrameRateFast
        {
            get { return Properties.Settings.Default.officeframeratefast; }
        }

        public double officeFrameRateSlow
        {
            get { return Properties.Settings.Default.officeframerateslow; }
        }

        public String officeMessage
        {
            get { return _transients[OFFICE_MESSAGE_NAME]; }
            set
            {
                _transients[OFFICE_MESSAGE_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(OFFICE_MESSAGE_NAME, value));
            }
        }

        public Constants.VIDEO_PROTOCOLS officeProtocol
        {
            get { return (Constants.VIDEO_PROTOCOLS)Properties.Settings.Default.officeprotocol; }
        }

        public String officeSkypeHandle
        {
            get { return _transients[OFFICE_SKYPE_HANDLE_NAME]; }
            set
            {
                _transients[OFFICE_SKYPE_HANDLE_NAME] = value;

                if (_objStream != null)
                    _objStream.Send(new ValuePacket(OFFICE_SKYPE_HANDLE_NAME, value));
            }
        }

        
        //
        // Public methods
        //
        public void ActuateDoor()
        {
            if (_objStream != null)
                _objStream.Send(new ValuePacket(HOME_DOOR_ACTUATE_NAME, Boolean.TrueString));
        }

        public void SetStream(IObjectStream objStream)
        {
            _objStream = objStream;
        }

        public void Populate()
        {
            if (_objStream == null)
                return;

            // Send transient values
            foreach (String name in _transients.Keys)
            {
                // Echo prevention
                _remotesToIgnore.Add(name);

                _objStream.Send(new ValuePacket(name, _transients[name]));
            }

            // Send configuration values
            foreach (SettingsProperty prop in Properties.Settings.Default.Properties)
            {
                String name = prop.Name;
                object value = Properties.Settings.Default.GetType().GetProperty(name).GetValue(Properties.Settings.Default, null);

                // Echo prevention
                _remotesToIgnore.Add(name);

                _objStream.Send(new ValuePacket(name, value));
            }
        }


        //
        // Change handlers (local and remote)
        //
        public void HandleIncomingMessage(Object payload)
        {
            if (payload is ValuePacket)
            {
                ValuePacket packet = (ValuePacket)payload;
                String valueString = (packet.value != null) ? packet.value.ToString() : null;

                // Handle door actuation separately (it's a special case)
                if (packet.key.Equals(HOME_DOOR_ACTUATE_NAME))
                {
                    if (valueString != null && valueString.Equals(Boolean.TrueString))
                    {
                        if (_objStream != null)
                            _objStream.Send(new ValuePacket(HOME_DOOR_ACTUATE_NAME, Boolean.FalseString));

                        if (DoorActuated != null)
                            DoorActuated();
                    }

                    return;
                }
                
                // Echo prevention
                if (_remotesToIgnore.Contains(packet.key))
                {
                    _remotesToIgnore.Remove(packet.key);
                    return;
                }

                // Store the value locally
                if (_transients.ContainsKey(packet.key))
                {
                    _transients[packet.key] = (packet.value == null) ? null : packet.value.ToString();
                }
                else
                {
                    _localsToIgnore.Add(packet.key);

                    Properties.Settings.Default.GetType().GetProperty(packet.key).SetValue(Properties.Settings.Default, packet.value, null);
                    Properties.Settings.Default.Save();
                }

                // Send out a notification if requested
                SendNotifications(packet.key);
            }
        }

        
        private void Default_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            String name = e.PropertyName;
            object value = Properties.Settings.Default.GetType().GetProperty(name).GetValue(Properties.Settings.Default, null);

            // Echo prevention
            if (_localsToIgnore.Contains(name))
            {
                _localsToIgnore.Remove(name);
                return;
            }

            // Skip the broadcast if we don't have anywhere to broadcast the new value to
            if (_objStream != null)
            {
                // Echo prevention
                _remotesToIgnore.Add(name);

                _objStream.Send(new ValuePacket(name, value));
            }

            // Send out a notification if requested
            SendNotifications(name);
        }

        //
        // Notification disptach helper
        //
        private void SendNotifications(String name)
        {
            if (DoorActuationAvailable != null &&
                name.Equals(DOOR_ACTUATION_AVAILABLE_NAME))
                DoorActuationAvailable(doorActuationAvailable);
            else if (DoorbellStatusChanged != null &&
                     name.Equals(DOORBELL_STATUS))
                DoorbellStatusChanged(doorbellStatus);
            else if (HomeVideoConfigurationChanged != null &&
                (name.Equals(HOME_COMPRESSION_NAME) ||
                 name.Equals(HOME_FRAME_RATE_FAST_NAME) ||
                 name.Equals(HOME_FRAME_RATE_SLOW_NAME) ||
                 name.Equals(HOME_PROTOCOL_NAME)))
                HomeVideoConfigurationChanged(homeProtocol, homeCompression, homeFrameRateSlow, homeFrameRateFast);
            else if (HomeMessageChanged != null &&
                name.Equals(HOME_MESSAGE_NAME))
                HomeMessageChanged(homeMessage);
            else if (HomeNoteTextChanged != null &&
                name.Equals(HOME_NOTE_TEXT_NAME))
                HomeNoteTextChanged(homeNoteText);
            else if (HomeSkypeHandleChanged != null &&
                name.Equals(HOME_SKYPE_HANDLE_NAME))
                HomeSkypeHandleChanged(homeSkypeHandle);
            else if (HomeStatusChanged != null &&
                name.Equals(HOME_STATUS_NAME))
                HomeStatusChanged(homeStatus);
            else if (OfficeVideoConfigurationChanged != null &&
                (name.Equals(OFFICE_COMPRESSION_NAME) ||
                 name.Equals(OFFICE_FRAME_RATE_FAST_NAME) ||
                 name.Equals(OFFICE_FRAME_RATE_SLOW_NAME) ||
                 name.Equals(OFFICE_PROTOCOL_NAME)))
                OfficeVideoConfigurationChanged(officeProtocol, officeCompression, officeFrameRateSlow, officeFrameRateFast);
            else if (OfficeMessageChanged != null &&
                name.Equals(OFFICE_MESSAGE_NAME))
                OfficeMessageChanged(officeMessage);
            else if (OfficeSkypeHandleChanged != null &&
                name.Equals(OFFICE_SKYPE_HANDLE_NAME))
                OfficeSkypeHandleChanged(officeSkypeHandle);
        }
    }
}
