﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;

namespace MEdiaSpaceHome
{
    public partial class ConnectDialog : Form
    {
        private const string NONE_LABEL = "(none)";     // Used to represent that there was no previous IP address
        private const int AUTOCONNECT_TIMEOUT = 15;     // Seconds before autoconnect
        private int autoconnectcount = 0;               // Amount of seconds already counted
        private IPAddress ip = null;                    // The selected IP address to connect to

        public ConnectDialog()
        {
            InitializeComponent();
            pbAutoconnect.Minimum = 0;
            pbAutoconnect.Maximum = AUTOCONNECT_TIMEOUT;
            pbAutoconnect.Value = autoconnectcount;

            // Load the last IP address that this user connected to, if it exists
            lblLastIP.Text = Properties.Settings.Default.LastIP != string.Empty ? Properties.Settings.Default.LastIP : NONE_LABEL;
            if (!lblLastIP.Text.Equals(NONE_LABEL)) tmrAutoconnect.Start();

            // Load the history list of IP addresses
            List<string> iphistory = new List<string>(Properties.Settings.Default.IPHistory.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries));
            cmbIP.Items.AddRange(iphistory.ToArray());
        }

        #region Properties
        public IPAddress SelectedIP
        {
            get
            {
                return ip;
            }
        }
        #endregion

        #region Event Handlers
        private void tmrAutoconnect_Tick(object sender, EventArgs e)
        {
            tmrAutoconnect.Stop();

            // Increment the timer and update the progress bar
            autoconnectcount++;
            pbAutoconnect.Value = autoconnectcount;

            // When timeout occurs...
            if (autoconnectcount >= AUTOCONNECT_TIMEOUT)
            {
                connectButton.PerformClick();
                return;
            }

            tmrAutoconnect.Start();
        }

        private void cmbIP_TextChanged(object sender, EventArgs e)
        {
            // Reset the autoconnect countdown
            autoconnectcount = 0;
            pbAutoconnect.Value = autoconnectcount;
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            this.Enabled = false;
            string connectip = "";

            // If no IP address text is entered...
            if (cmbIP.Text.Trim() == string.Empty)
            {
                // Default to the last IP address, if possible
                if (Properties.Settings.Default.LastIP != string.Empty)
                {
                    connectip = Properties.Settings.Default.LastIP;
                }
                else
                {
                    MessageBox.Show("You must specify an IP address to connect to!", "IP Address Required", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    this.Enabled = true;
                    return;
                }
            }
            else connectip = cmbIP.Text.Trim();

            // Validate the IP address
            if (IPAddress.TryParse(connectip, out ip))
            {
                DialogResult = DialogResult.OK;
                return;
            }
            else
            {
                MessageBox.Show("You must specify a valid IP address to connect to!", "Invalid IP Address", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                this.Enabled = true;
                return;
            }
        }
        #endregion
    }
}
